---
---  \file wrap_array.lua
---  \brief Lua  wrapper for Array classes
--- 
--- This module mainly organizes access to Arrays using
--- "normal" array syntax in Lua, namely
--- \c a[i] for a one-dimensional array and
--- \c a[i][j] for a two-dimensional array.
--- 
--- Furthermore, various constructors are defined.
--- \todo Hartmuts Arrays, dann dokumentieren.

-- lua array type, currently for example only (lua written linenhandler)


---------------------------------------------------
--- \brief
--- \luaonly Create a Lua  table as array 
---
--+ Array* Array2(table desc);


function Array2(desc)
   local a=desc
   a.linehandler=function(x,l,...) x[l]=arg  end
   a.dim=2
   return a
end
      
---------------------------------------------------
--- \brief
--- \luaonly Create a Lua  table as array 
--+ Array *Array1(table desc);

function Array1(desc)
   local a=desc
   a.linehandler=function(x,l,...) x[l]=arg[1]  end
   a.dim=1
   return a
end


---------------------------------------------------
--- \brief
--- \luaonly Return ith row of array.
--+ Array *row(Array *array, int i);
function row(array,i)
   if a.dim==1 then return array[i] end 
   return array:row(i)
end


---------------------------------------------------
--- \brief
--- \luaonly Return ith column of array
--+ Array*  col(Array* array,int i);

function col(array,i)
   if a.dim==1 and i==1 then return array end 
   if a.dim==1 then return nil end 
   return array:col(i)
end


---------------------------------------------------
--- \brief
--- \luaonly Return "flattened" array.
--- \see da1All
--+ Array *all(Array *array);
function all(array)
   if a.dim==1 then return array
   elseif  a.basetype=="int" then return ia2All(array)
   else return mbase.da2All(array)
   end
end



if not _array1 then _array1={} end
if not _array2 then _array2={} end
_array1["lua"]=Array1
_array2["lua"]=Array2

-- array constructor.
-- basically, this reads desc.dim and desc.type and
-- passes to _T_arrayX

---------------------------------------------------
--- \brief
--- \luaonly Description table based constructor for generic  arrays
--- 
--- Can be used like
--- \verbatim 
--- a=Array{type="double",length=n} 
--- \endverbatim
---
--+ Array *Array (ArrayClass *self,ArrayDescriptionTable *desc);


----------------------------------------------------
---  \brief
---   \luaonly  Description table for Lua arrays
--- 
---   An array as description table  generates a copy of itself.
---   All elements in the table are optional.
---
--+  class  ArrayDescriptionTable{
--+    public:
--+   char* type; /**< Type name. Default: "double"*/
--+   int dim; /**< Dimension. Default: 1.*/
--+   int n;   /**< Length. Default: 0.*/
--+   int nrow; /**< Number of rows. Default:0 */
--+   int ncol; /**< Number of columns. Default: 0*/
--+   char * file; /**< file to be read from (for optional data statement). Default: nil.*/
--+   char * delim; /**< token delimiters  (for optional data statement). Default: nil.*/
--+   char * string; /**< data string (for optional data statement). Default: nil.*/
--+ }


function Array(...)
   local a,basetype,dim
   local desc=arg[1]
   -- no description - no output
   if not desc then return nil end 
   
   basetype=desc.type
   if not basetype then basetype="double" end 

   dim=desc.dim
   if not dim  then
     if desc.file or desc.data then 
       dim=2 
     else
       dim=1
     end
    end
   
    if istype(desc,"DArray1") or  istype(desc,"DArray2") or  istype(desc,"IArray1") or  istype(desc,"IArray2") then 
      -- if desc is a typed array return it.
      a=desc 
      
   elseif type(desc)~="table" then 
      -- from here on, we need a valid table as description
      return nil
   
   elseif table.getn(desc)>0 and type(desc[1])=="table" and table.getn(desc[1])>0 then
      -- 2D array in Lua syntax
      a=_array2[basetype](desc) 
   elseif table.getn(desc)>0 then
      -- 1D array in Lua syntax
      a= _array1[basetype](desc) 
   elseif dim==1 then  
      -- Empty 1D array
      a=_array1[basetype](desc) 
   elseif   dim==2 then
      -- Empty 2D array
      a= _array2[basetype](desc) 
   end
   if desc.file or desc.string then 
      return Data{a; file=desc.file, string=desc.string, delim=desc.delim}
   end
   return a
end

function linspace(xbeg,xend,incr)
 local n=math.ceil(math.abs((xend-xbeg)/incr))
 local h=(xend-xbeg)/n
 local a=mbase.da1Create(n+1)
 local i=1
 local x=xbeg
 for i=1,n+1 do 
   mbase.da1Set(a,i,x)
   x=x+h
  end
  return a,n+1
end

function getn(a)
   if type(a)=="table" then return table.getn(a) end
   return vdim(a)
end

--  function ia1Check(a) if %ia1Check(a)==0 then return nil else return 1 end end
--  function ia2Check(a) if %ia2Check(a)==0 then return nil else return 1 end end
--  function da1Check(a) if %da1Check(a)==0 then return nil else return 1 end end
--  function da2Check(a) if %da2Check(a)==0 then return nil else return 1 end end
___x=nil  




